# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from ..shared.image_object import ImageObject

__all__ = ["ImageListResponse"]

ImageListResponse: TypeAlias = List[ImageObject]
